/*
 * @(#)BufferedReader.java	1.11 97/03/03
 * 
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.1_beta
 * 
 */

package extranet;

import java.io.BufferedReader;
// import java.io.BufferedInputStream;
import java.io.Reader;
import java.io.InputStream;

import java.io.IOException;

public class MUDBufferedReader extends BufferedReader {
// public class MUDBufferedReader extends BufferedInputStream {

    private Reader in;
    // private InputStream in;

    private char cb[];
    // private byte cb[];
    private int nChars, nextChar;

    private static final int INVALIDATED = -2;
    private static final int UNMARKED = -1;
    private int markedChar = UNMARKED;
    private int readAheadLimit = 0; /* Valid only when markedChar > 0 */

    private static int defaultCharBufferSize = 8192;
    private static int defaultExpectedLineLength = 80;
    private long prompt_sec = 0;  // number of milliseconds we've waited since the last character
    private static int prompt_usec = 100;      // Max wait time for a new character

    /**
     * Create a buffering character-input stream that uses an input buffer of
     * the specified size.
     *
     * @param  in   A Reader
     * @param  sz   Input-buffer size
     *
     * @exception  IllegalArgumentException  If sz is <= 0
     */
    public MUDBufferedReader(Reader in, int sz) {
    // public MUDBufferedReader(InputStream in, int sz) {
        // super(in);
        super(in, sz);
	if (sz <= 0)
	    throw new IllegalArgumentException("Buffer size <= 0");
	this.in = in;
        cb = new char[sz];
        // cb = new byte[sz];
	nextChar = nChars = 0;
    }

    /**
     * Create a buffering character-input stream that uses a default-sized
     * input buffer.
     *
     * @param  in   A Reader
     */
    public MUDBufferedReader(Reader in) {
        // public MUDBufferedReader(InputStream in) {
        this(in, defaultCharBufferSize);
    }

    /** Check to make sure that the stream has not been closed */
    private void ensureOpen() throws IOException {
	if (in == null)
	    throw new IOException("Stream closed");
    }

    /**
     * Fill the input buffer, taking the mark into account if it is valid.
     */
    private void fill() throws IOException {
        int dst;
        if (markedChar <= UNMARKED) {
            /* No mark */
            dst = 0;
        } else {
            /* Marked */
            int delta = nextChar - markedChar;
            if (delta >= readAheadLimit) {
                /* Gone past read-ahead limit: Invalidate mark */
                markedChar = INVALIDATED;
                readAheadLimit = 0;
                dst = 0;
            } else {
                if (readAheadLimit <= cb.length) {
                    /* Shuffle in the current buffer */
                    System.arraycopy(cb, markedChar, cb, 0, delta);
                    markedChar = 0;
                    dst = delta;
		} else {
                    /* Reallocate buffer to accomodate read-ahead limit */
                    char ncb[] = new char[readAheadLimit];
                    // byte ncb[] = new byte[readAheadLimit];
                    System.arraycopy(cb, markedChar, ncb, 0, delta);
                    cb = ncb;
                    markedChar = 0;
                    dst = delta;
                }
            }
        }

        int n;
        do {
            n = in.read(cb, dst, cb.length - dst);
        } while (n == 0);
        if (n > 0) {
            nChars = dst + n;
            nextChar = dst;
        }
    }

    /**
     * Read a line of text.  A line is considered to be terminated by any one
     * of a line feed ('\n'), a carriage return ('\r'), or a carriage return
     * followed immediately by a linefeed.
     *
     * @return     A String containing the contents of the line, not including
     *             any line-termination characters, or null if the end of the
     *             stream has been reached
     *
     * @exception  IOException  If an I/O error occurs
     */
    public String readLine() throws IOException {
	StringBuffer s = new StringBuffer(defaultExpectedLineLength);
        synchronized (lock) {
            // synchronized (in) {
	    ensureOpen();

	bufferLoop:
	    for (;;) {

                if (nextChar >= nChars) {
                    // if (!in.ready() && s.length() > 0) {
                    prompt_sec = System.currentTimeMillis();
                    while (!in.ready() && s.length() > 0) {
                        if (System.currentTimeMillis() - prompt_sec >= prompt_usec) {
                            System.out.println("Time: " + (System.currentTimeMillis() - prompt_sec));
//                            char check = s.charAt(s.length() - 1);
//                            System.out.println("Checkchar: " + check);
//                            if (check == '\r' || check == '\n') {
//                                System.out.println("Cut down string in MUDBufferedReader");
//                                s.setLength(s.length() - 1);
//                            }
                            System.out.println("Returning string: *" + s.toString() + "*");
                            if (s.length() > 0) {
                                // We don't want to return a totally empty string
                                return s.toString();
                            } else {
                                System.out.println("Zero-length string");
                            }
                        }
                    }

                    fill();
                }
		if (nextChar >= nChars) { /* EOF */
                    if (s.length() > 0) {
                        System.out.println("MUDBufferedReader EOF");
                        return s.toString();
                    }
                    else {
//                        System.out.println("MUDBufferedReader returned null.");
                        return null;
                    }
		}
		boolean eol = false;
		char c = 0;
		int i;

//                System.out.println("Entering charLoop.");
	    charLoop:
		for (i = nextChar; i < nChars; i++) {
                    c = cb[i];
                    // c = (char)cb[i];
//                    System.out.print(c + "" + (int)c);
//                    System.out.print("1");
                    if ((c == '\n') || (c == '\r')) {
//                        System.out.print("*EOL*");
			eol = true;
			break charLoop;
		    }
//                    System.out.println(c + " -> " + cb[i]);
		}
                s.append(cb, nextChar, i - nextChar);
//                System.out.print("3");
                // for (int l = nextChar; l < i; l++) {
                // s.append((char)cb[l]);
                //}
                nextChar = i;

                if (eol) {
//                    System.out.print("4");
		    nextChar++;
                    //		    if (c == '\r') {
                    if (c == '\r' || c== '\n') {
                        if (nextChar >= nChars) {
//                            System.out.println("Calling a new fill, we seem to have run out of characters.");
                            fill();
                        }
//                        if ((nextChar < nChars) && (cb[nextChar] == '\n')) {
                            // Let's do a loop, incase we get sent some strange
                            // collection of carraige returns and linefeeds
                        // do {
                        while ((cb[nextChar] == '\n' || cb[nextChar] == '\r') && nextChar < nChars) {
//                            System.out.print("*" + (int)cb[nextChar]);
                            nextChar++;
                        }
//                            } while ((cb[nextChar] == '\n' || cb[nextChar] == '\r') && nextChar < nChars);
//                        }
		    }
                    break bufferLoop;
                }
            }
	}

        return s.toString();
    }

}
